exports.tpl = (ck_options) ->
  ck_buffer = []

  ck_render_attrs = (obj) ->
    str = ''
    for k, v of obj
      str += " #{k}=\"#{v}\""
    str

  ck_doctypes =
    '5': '<!DOCTYPE html>'
    'xml': '<?xml version="1.0" encoding="utf-8" ?>'
    'default': '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">'
    'transitional': '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">'
    'strict': '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">'
    'frameset': '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Frameset//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-frameset.dtd">'
    '1.1': '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">',
    'basic': '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML Basic 1.1//EN" "http://www.w3.org/TR/xhtml-basic/xhtml-basic11.dtd">'
    'mobile': '<!DOCTYPE html PUBLIC "-//WAPFORUM//DTD XHTML Mobile 1.2//EN" "http://www.openmobilealliance.org/tech/DTD/xhtml-mobile12.dtd">'
  ck_self_closing = 'area|base|basefont|br|hr|img|input|link|meta'.split '|'
    
  doctype = (type) ->
    type ?= 5
    text ck_doctypes[type]
    text '\n' if ck_options.format
    
  text = (txt) -> ck_buffer.push String(txt); null
  
  tag = (name, opts) ->
    text "<#{name}"
  
    for o in opts
      text ck_render_attrs(o) if typeof o is 'object'
  
    if name in ck_self_closing
      text ' />'
      text '\n' if ck_options.format
    else
      text '>'
      text '\n' if ck_options.format
  
      for o in opts
        switch typeof o
          when 'string', 'number'
            text o
            text '\n' if ck_options.format
          when 'function'
            result = o.call ck_options.context
            if typeof result is 'string'
              text result 
              text '\n' if ck_options.format
      text "</#{name}>"
      text '\n' if ck_options.format
  
    null
  
  coffeescript = (code) ->
    script ";(#{code})();"

  a = -> tag 'a', arguments
  abbr = -> tag 'abbr', arguments
  acronym = -> tag 'acronym', arguments
  address = -> tag 'address', arguments
  applet = -> tag 'applet', arguments
  area = -> tag 'area', arguments
  article = -> tag 'article', arguments
  aside = -> tag 'aside', arguments
  audio = -> tag 'audio', arguments
  b = -> tag 'b', arguments
  base = -> tag 'base', arguments
  basefont = -> tag 'basefont', arguments
  bdo = -> tag 'bdo', arguments
  big = -> tag 'big', arguments
  blockquote = -> tag 'blockquote', arguments
  body = -> tag 'body', arguments
  br = -> tag 'br', arguments
  button = -> tag 'button', arguments
  canvas = -> tag 'canvas', arguments
  caption = -> tag 'caption', arguments
  center = -> tag 'center', arguments
  cite = -> tag 'cite', arguments
  code = -> tag 'code', arguments
  col = -> tag 'col', arguments
  colgroup = -> tag 'colgroup', arguments
  command = -> tag 'command', arguments
  datalist = -> tag 'datalist', arguments
  dd = -> tag 'dd', arguments
  del = -> tag 'del', arguments
  details = -> tag 'details', arguments
  dfn = -> tag 'dfn', arguments
  dir = -> tag 'dir', arguments
  div = -> tag 'div', arguments
  dl = -> tag 'dl', arguments
  dt = -> tag 'dt', arguments
  em = -> tag 'em', arguments
  embed = -> tag 'embed', arguments
  fieldset = -> tag 'fieldset', arguments
  figcaption = -> tag 'figcaption', arguments
  figure = -> tag 'figure', arguments
  font = -> tag 'font', arguments
  footer = -> tag 'footer', arguments
  form = -> tag 'form', arguments
  frame = -> tag 'frame', arguments
  frameset = -> tag 'frameset', arguments
  h1 = -> tag 'h1', arguments
  h2 = -> tag 'h2', arguments
  h3 = -> tag 'h3', arguments
  h4 = -> tag 'h4', arguments
  h5 = -> tag 'h5', arguments
  h6 = -> tag 'h6', arguments
  head = -> tag 'head', arguments
  header = -> tag 'header', arguments
  hgroup = -> tag 'hgroup', arguments
  hr = -> tag 'hr', arguments
  html = -> tag 'html', arguments
  i = -> tag 'i', arguments
  iframe = -> tag 'iframe', arguments
  img = -> tag 'img', arguments
  input = -> tag 'input', arguments
  ins = -> tag 'ins', arguments
  keygen = -> tag 'keygen', arguments
  kbd = -> tag 'kbd', arguments
  label = -> tag 'label', arguments
  legend = -> tag 'legend', arguments
  li = -> tag 'li', arguments
  link = -> tag 'link', arguments
  map = -> tag 'map', arguments
  mark = -> tag 'mark', arguments
  menu = -> tag 'menu', arguments
  meta = -> tag 'meta', arguments
  meter = -> tag 'meter', arguments
  nav = -> tag 'nav', arguments
  noframes = -> tag 'noframes', arguments
  noscript = -> tag 'noscript', arguments
  object = -> tag 'object', arguments
  ol = -> tag 'ol', arguments
  optgroup = -> tag 'optgroup', arguments
  option = -> tag 'option', arguments
  output = -> tag 'output', arguments
  p = -> tag 'p', arguments
  param = -> tag 'param', arguments
  pre = -> tag 'pre', arguments
  progress = -> tag 'progress', arguments
  q = -> tag 'q', arguments
  rp = -> tag 'rp', arguments
  rt = -> tag 'rt', arguments
  ruby = -> tag 'ruby', arguments
  s = -> tag 's', arguments
  samp = -> tag 'samp', arguments
  script = -> tag 'script', arguments
  section = -> tag 'section', arguments
  select = -> tag 'select', arguments
  small = -> tag 'small', arguments
  source = -> tag 'source', arguments
  span = -> tag 'span', arguments
  strike = -> tag 'strike', arguments
  strong = -> tag 'strong', arguments
  style = -> tag 'style', arguments
  sub = -> tag 'sub', arguments
  summary = -> tag 'summary', arguments
  sup = -> tag 'sup', arguments
  table = -> tag 'table', arguments
  tbody = -> tag 'tbody', arguments
  td = -> tag 'td', arguments
  textarea = -> tag 'textarea', arguments
  tfoot = -> tag 'tfoot', arguments
  th = -> tag 'th', arguments
  thead = -> tag 'thead', arguments
  time = -> tag 'time', arguments
  title = -> tag 'title', arguments
  tr = -> tag 'tr', arguments
  tt = -> tag 'tt', arguments
  u = -> tag 'u', arguments
  ul = -> tag 'ul', arguments
  video = -> tag 'video', arguments
  xmp = -> tag 'xmp', arguments

  ck_tpl = ->
    text @title    
    doctype 5
    html lang: 'en', ->
      head ->
        meta charset: 'utf-8'
        title @title
        style '''
          body {font-family: "sans-serif"}
          section, header {display: block}
        '''
      body ->
        section ->
          header ->
            h1 @title
          if @inspired
            p 'Create a witty example'
          else
            p 'Go meta'
          ul ->
            for user in @users
              li user.name
              li -> a href: "mailto:#{user.email}", -> user.email

  ck_tpl.call ck_options.context
  
  ck_buffer.join ''