class Test
  constructor: ->
    buffer = ''
    
    @context = {foo: 'bar'}

    @locals =
      render: (txt) -> buffer += txt + '!'
      send: (txt) -> buffer += txt + '?'
      redirect: (txt) -> buffer += txt + '$'
      ping: 'pong'

  unwrap: (code) ->
    code = String(code)
    if code.search /^(\s)*function/ > -1
      code = code.replace /^(\s)*function(\s)*\(.*\)(\s)*\{/, ''
      code = code.replace /\}(\s)*$/, ''

  benchmark: (title, code) ->
    start = new Date
    for i in [1..400000]
      code()
    puts "#{title}: #{new Date - start} ms"

  handler: (ping) ->
    render ping
    render @foo
    send @foo
    redirect @foo

  change_signature: (hand) ->
    String(hand).replace '(ping)', '(render, send, redirect, ping)'

  build_vars: ->
    vars = []
    `for(k in this.locals) {vars.push('var ' + k + ' = locals.' + k + ';')}`
    vars.join ''


  go_with: ->
    func = new Function 'locals', "with(locals) {#{@unwrap @handler}}"
    @benchmark 'with', => func.call @context, @locals

  go_hardcoded: ->
    func = new Function 'render', 'send', 'redirect', 'ping', @unwrap(@handler)
    @benchmark 'hardcoded', => func.apply @context, [@locals.render, @locals.send, @locals.redirect, @locals.ping]

  go_hardcoded_apply: ->
    func = new Function 'context', 'render', 'send', 'redirect', "(#{@handler}).apply(context, [arguments[4]])"
    @benchmark 'hardcoded apply', => func @context, @locals.render, @locals.send, @locals.redirect, @locals.pong

  go_signature: ->
    func = new Function 'context', 'locals', "(#{@change_signature @handler}).apply(context, locals)"
    @benchmark 'signature', => func @context, [@locals.render, @locals.send, @locals.redirect, @locals.ping]

  go_vars: ->
    func = new Function 'locals', @build_vars() + @unwrap(@handler)
    @benchmark 'vars', => func.call @context, @locals

new Test().go_vars()
new Test().go_hardcoded()
new Test().go_hardcoded_apply()
new Test().go_signature()
new Test().go_with()
